Ext.define("demo.view.theme.MainController", {
   extend : "Ext.app.ViewController",
   alias  : "controller.demoThemeMainController",
    
    
   init : function () {
      this.initGridDataStore();
   },
    
    
   sendNotification : function () {
      const vm = this.getViewModel();
      const notificationText = `${Ext.util.Format.date(new Date(), "H:i:s")  } ${  vm.get("notificationText")}`;
      let toastTarget = undefined;
      if (vm.get("notificationToastTarget") && vm.get("notificationToastTarget").notificationToastTarget === "module") {
         toastTarget = this.getView();
      } else if (vm.get("notificationToastTarget") && vm.get("notificationToastTarget").notificationToastTarget === "fieldset") {
         toastTarget = this.getView().down("#notificationFieldSet");
      }
        
      BpcCommon.Api.showNotification({
         text        : notificationText,
         type        : vm.get("notificationType").notificationType,
         popUp       : vm.get("notificationPopUp"),
         toast       : vm.get("notificationToast"),
         toastTarget : toastTarget,
         silent      : vm.get("notificationSilent")
      });
   },
    
   initGridDataStore : function () {
      Ext.create("Ext.data.Store", {
         storeId : "simpsonsStore",
         fields  : ["name", "email", "phone"],
         data    : [
            { name: "Lisa", email: "lisa@simpsons.com", phone: "555-111-1224" },
            { name: "Bart", email: "bart@simpsons.com", phone: "555-222-1234" },
            { name: "Homer", email: "homer@simpsons.com", phone: "555-222-1244" },
            { name: "Marge", email: "marge@simpsons.com", phone: "555-222-1254" }
         ]
      });
   },
    
   showLoadingMask : function () {
      const maskTarget = this.getView().down("#maskedPanel");
      new Ext.LoadMask({ msg: "Please wait...", target: maskTarget }).show();
   },
    
   openGridCellEditor : function (button) {
      if (!button.modiwin) {
         button.modiwin = this.createCellEditor();
      }
      button.modiwin.on("close", function () {
         // ensure the old window is cleaned out (in case its close
         // action just hides it)
         button.modiwin.destroy();
        
         // remember we don't have a window anymore
         button.modiwin = null;
      });
      button.modiwin.show();
   },
    
   openGridRowEditor : function (button) {
      if (!button.modiwin) {
         button.modiwin = this.createRowEditor();
      }
      button.modiwin.on("close", function () {
         // ensure the old window is cleaned out (in case its close
         // action just hides it)
         button.modiwin.destroy();
        
         // remember we don't have a window anymore
         button.modiwin = null;
      });
    
      button.modiwin.show();
   },
    
   createCellEditor : function () {
      return  Ext.create("Ext.window.Window", {
         title : "Cell Editor Demo",
         cls   : "grid-cell-edit-demo",
         items : [
            Ext.create("Ext.grid.Panel", {
               store   : Ext.data.StoreManager.lookup("simpsonsStore"),
               columns : [
                  { header: "Name", dataIndex: "name", editor: "textfield" },
                  { header    : "Email", dataIndex : "email", flex      : 1,
                     editor    : {
                        xtype      : "textfield",
                        allowBlank : false
                     } },
                  { header: "Phone", dataIndex: "phone" }
               ],
               selModel : "cellmodel",
               plugins  : {
                  ptype        : "cellediting",
                  clicksToEdit : 1
               },
               height : 200,
               width  : 500
            })
         ]
      });
   },
    
   createRowEditor : function () {
      return Ext.create("Ext.window.Window", {
         title : "Row Editor Demo",
         cls   : "grid-row-edit-demo",
         items : [
            Ext.create("Ext.grid.Panel", {
               store   : Ext.data.StoreManager.lookup("simpsonsStore"),
               columns : [
                  { header: "Name", dataIndex: "name", editor: "textfield" },
                  { header    : "Email", dataIndex : "email", flex      : 1,
                     editor    : {
                        xtype      : "textfield",
                        allowBlank : false
                     } },
                  { header: "Phone", dataIndex: "phone" }
               ],
               selModel : "rowmodel",
               plugins  : {
                  rowediting : {
                     clicksToEdit : 1
                  }
               },
               height : 200,
               width  : 500
            })
         ]
      });
   },
    
    
   /**
     * All the functions for card navigation
     */
   showNext : function () {
      this.doCardNavigation(1);
   },
    
   showPrevious : function () {
      this.doCardNavigation(-1);
   },
    
   doCardNavigation : function (incr) {
      const me = this.getView(),
            l = me.getLayout(),
            i = l.activeItem.id.split("card-")[1],
            next = parseInt(i, 10) + incr;
        
      l.setActiveItem(next);
        
      me.down("#card-prev").setDisabled(next === 0);
      me.down("#card-next").setDisabled(next === 2);
   },

   openGridJsonEditorColumn : function () {
      Ext.create("Ext.window.Window", {
         title : "Grid with JSON Editor Column Demo",
         cls   : "grid-row-edit-demo",
         items : [
            {
               xtype : "demoGridWithJsonColumn"
            }
         ]
      }).show();
   }

});