/**
 * Widget which used the configuration management from the bpc widget mixin
 * Created by bnz on 31.08.17
 */
Ext.define("demo.view.widget.demoWidget", {
   extend : "Ext.container.Container",
   alias  : "widget.demoWidget",

   /**
     * include mixin to get widget configuration management
     */
   mixins : [
      "BpcCommon.mixin.Widget"
   ],

   /**
     * some information about this widget
     */
   statics : {
      WIDGET_NAME        : "demo Hello World Widget",
      WIDGET_DESCRIPTION : "Hello World Description",
      WIDGET_ICON_CLS    : "x-fa fa-bug"
   },

   layout      : "card",
   activeItems : 0,

   /**
     * here comes all manages settings with types
     * for default values se config.widgetConfiguration
     */
   widgetConfigSettingMap : {
      managedSettingWithDefault    : { settingType: "text" },
      managedSettingWithOUTDefault : { settingType: "number" } // no defaul defined in config.widgetConfiguration
   },

   /**
     * here comes the default values for settings
     * settings which are not in widgetConfigSettingMap are unmanaged
     * if a value changes the event widgetConfigurationChange will be fired by the mixin
     */
   config : {
      widgetConfiguration : {
         unmanagedSetting          : "value", // unmanaged -> not listed in widgetConfigSettingMap
         managedSettingWithDefault : "default of managedSettingWithDefault" // default for widgetConfigSettingMap.managedSettingWithDefault
      }
   },

   /**
     * only for demo
     */
   viewModel : {
      data : {
         vmDataUnmanagedSetting             : "nothing1",
         vmDataManagedSettingWithDefault    : "nothing2",
         vmDataManagedSettingWithOUTDefault : "nothing3"
      }
   },

   initComponent : function () {
      Ext.log({ msg: `${this.self.getName()  }: initComponent`, level: "info" });

      // have to call this to get configuration managed
      this.initWidgetSettings();

      this.callParent();

      // add content
      this.content = this.add({
         xtype    : "container",
         layout   : "form",
         defaults : {
            xtype    : "textfield",
            readOnly : true
         },
         items : [{
            fieldLabel : "unmanagedSetting",
            bind       : "{vmDataUnmanagedSetting}"
         }, {
            fieldLabel : "managedSettingWithDefault",
            bind       : "{vmDataManagedSettingWithDefault}"
         }, {
            fieldLabel : "managedSettingWithOUTDefault",
            bind       : "{vmDataManagedSettingWithOUTDefault}"
         }]
      });

      /*
        add BPC Setting Panel to the the content of settingStore
        the settingStore will be created by the mixin and contains all managed settings
         */
      this.settingPanel = this.add({
         xtype    : "bpcSettingPanel",
         settings : this.settingStore,
         buttons  : [
            {
               text    : "OK",
               handler : this.setActiveItem.bind(this, 0)
            }
         ]
      });

      // init viemodel data
      this.initWidget();
   },

   /**
     * Placeholderfunction. This method will be called by external component like dashboard to refresh the widget
     * data/view.
     */
   refreshWidget : function () {
      Ext.log({
         msg   : `${this.self.getName()  }: refreshWidget not implemented for this widget`,
         level : "warn"
      });
   },

   /**
     * this function will be called by dashboard when the user clicks on the cog icon
     */
   configureWidget : function () {
      Ext.log({ msg: `${this.self.getName()  }: configureWidget`, level: "info" });
      this.setActiveItem(1);
   },

   /**
     * will be called every time the managed configuration was changed
     * implementation is optional
     */
   initWidget : function () {
      Ext.log({ msg: `${this.self.getName()  }: initWidget`, level: "info" });
      const vm = this.getViewModel();

      /*
        update content via data binding
        the function getSetting came from the mixin and returns the current value of a setting
         */
      vm.set("vmDataUnmanagedSetting", this.getSetting("unmanagedSetting"));
      vm.set("vmDataManagedSettingWithDefault", this.getSetting("managedSettingWithDefault"));
      vm.set("vmDataManagedSettingWithOUTDefault", this.getSetting("managedSettingWithOUTDefault"));
   }
});